
#include <stdio.h>
#include "global.h"
#include "processor.h"
#include "list.h"
#include "dag.h"
#include "solve.h"


int
main( int argc, char *argv[] )
{
    int	     length;	// number of cycles used by scheduler to schedule DAG
    DAG_type *DAG;	// data dependencies DAG
    int      *schedule;	// scheduling order of each instruction
    void     printSchedule( DAG_type *DAG, int schedule[] );

    if( argc != 3 ) {
	fprintf( stderr, "Usage: schedule abcd problem\n" );
	fprintf( stderr, "\ta -- number of integer functional units\n" );
	fprintf( stderr, "\tb -- number of floating point functional units\n" );
	fprintf( stderr, "\tc -- number of load/store units\n" );
	fprintf( stderr, "\td -- number of branching units\n" );
	exit( 6 );
    }

    setFunctionalUnits( argv[1] );

    DAG = getDAG( argv[2] );

    schedule = (int *)calloc( (size_t)DAG->nInstructions, sizeof(int) );

    //length = listScheduler( DAG, schedule );
    length = optimalScheduler( DAG, schedule );

    //printSchedule( DAG, schedule );

    free( schedule );

    return( 0 );
}

void
printSchedule( DAG_type *DAG, int schedule[] )
{
    int  time, i, m;

    m = schedule[DAG->nInstructions-1];
    for( time = 1; time <= m; time++ ) {
	printf( "Instruction(s) at time %d:", time );
	for( i = 0; i < DAG->nInstructions; i++ ) {
	    if( schedule[i] == time ) {
		   printf( " %d(%d)", i, DAG->instructionType[i] );
	    }
	}
	printf( "\n" );
    }
}

