
#include <string.h>
#include "global.h"
#include "processor.h"

/* =================================================================
 *  Routines for answering queries about the processor model
 */

/*
 *  Number of functional units of each instruction type.
 *  The order of the values corresponds to the order of
 *  the instruction types below.
 *  Assumption: The issue width of the machine is the total
 *  of the number of functional units.
 */
static int n_func_units[N_INSTRUCTION_TYPES] = {
    1,		// number of integer functional units
    1,		// number of floating point functional units
    1,		// number of load/store units
    1,		// number of branching units
    HUGE,	// number of units for compiler pseudo-instructions
};

/*
 *  Names of instructions types as they appear in the input files.
 *  Internally the names are 0, 1, ...
 *  Assumption: DUMMY instruction is always listed last since this is
 *  assumed by propagator (the resource constraint is not propagated on
 *  the last instruction type). Even if there are no DUMMY instructions,
 *  they should be included.
 */
static char *instruction_type[N_INSTRUCTION_TYPES] = {
    "FXU",	// integer instructions
    "FPU",	// floating point instructions
    "LSU",	// load or store instruction
    "BPU",	// branching instructions
    "DUMMY"	// compiler directive pseudo-instructions
};

int
getInstructionType( char *line )
{
    int i;

    if( strcmp( line, "FXU+BPU" ) == 0 ) {
	strcpy( line, "FXU" );
    }
    else
    if( strcmp( line, "FPU+BPU" ) == 0 ) {
	strcpy( line, "FPU" );
    }
    for( i = 0; i < N_INSTRUCTION_TYPES; i++ ) {
	if( strcmp( line, instruction_type[i] ) == 0 ) {
	    break;
	}
    }
    if( i == N_INSTRUCTION_TYPES ) {
	fprintf( stderr, "Instruction type not found: %s\n", line );
	exit( 6 );
    }
/*
 *  Single-issue.
 */
/*
    if( (i == INTEGER_INSTR) ||
	(i == FLOAT_INSTR) ||
	(i == LDST_INSTR) ||
	(i == BRANCH_INSTR) ) {
	return(INTEGER_INSTR);
    }
    else {
	return(i); // PSEUDO_INSTR
    }
*/
/*
 *  Double-issue.
 */
/*
    if( (i == INTEGER_INSTR) ||
	(i == LDST_INSTR)    ||
	(i == BRANCH_INSTR) ) {
	return(INTEGER_INSTR);
    }
    else {
	return(i); // FLOAT_INSTR or PSEUDO_INSTR
    }
*/
/*
 *  4- or 6-issue.
 */
    return( i );
/*
*/
}

void
setFunctionalUnits( char *units )
{
    n_func_units[INTEGER_INSTR] = units[INTEGER_INSTR] - '0';
    n_func_units[FLOAT_INSTR  ] = units[FLOAT_INSTR  ] - '0';
    n_func_units[LDST_INSTR   ] = units[LDST_INSTR   ] - '0';
    n_func_units[BRANCH_INSTR ] = units[BRANCH_INSTR ] - '0';
}

int
nFunctionalUnits( int type )
{
    return( n_func_units[type] );
}

