
/* =================================================================
 *  Routines for getting the amount of CPU time used.
 */

#include <stdio.h>
#include <sys/time.h>
#include <sys/resource.h>
#include "global.h"
#include "processor.h"
#include "list.h"
#include "dag.h"
#include "csp.h"

static struct rusage res;

/*
 *  Start the stopwatch.
 */
void
set_start_time( CSP_type *C )
{
    getrusage( RUSAGE_SELF, &res );
    C->startTime = (double) res.ru_utime.tv_sec +
                   (double) res.ru_stime.tv_sec +
                   (double) res.ru_utime.tv_usec / 1000000.0 +
                   (double) res.ru_stime.tv_usec / 1000000.0;
}

/*
 *  Return the CPU time used in seconds.
 */
double
elapsed_time( CSP_type *C )
{
    getrusage( RUSAGE_SELF, &res );
    return( (double) res.ru_utime.tv_sec +
            (double) res.ru_stime.tv_sec +
            (double) res.ru_utime.tv_usec / 1000000.0 +
            (double) res.ru_stime.tv_usec / 1000000.0
            - C->startTime );
}

/* =================================================================
 *  Routines for setting and testing limits on the amount of CPU
 *  time used.
 */

/*
 *  Set an absolute limit on the CPU time, as
 *  measured from the creation time of the CSP.
 */
void
set_absolute_CPU_limit( CSP_type *C, double time_limit )
{
    C->maxCPUseconds = time_limit;
}

/*
 *  Set a relative limit on the CPU time, as
 *  measured from the current time.
 */
void
set_relative_CPU_limit( CSP_type *C, double time_limit )
{
    C->maxCPUseconds = elapsed_time( C ) + time_limit;
}

/*
 *  Answer whether the preset limit has been exceeded.
 */
BOOL
limit_exceeded( CSP_type *C )
{
    return( elapsed_time( C ) > C->maxCPUseconds );
}

